"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.executableOpenAPIMiddlewarePathParameters = void 0;
const openapi_document_local_ref_resolver_1 = require("openapi-document-local-ref-resolver");
const uriTemplates = require("uri-templates");
const ts_invariant_1 = require("ts-invariant");
const memoize = require("memoizee");
const _2020_1 = require("ajv/dist/2020");
const executable_openapi_parameters_1 = require("executable-openapi-parameters");
const executableOpenAPIMiddlewarePathParameters = ({ documentRefResolver, formatErrorResponse = executable_openapi_parameters_1.formatInvalidParameterResponse, formats } = {}) => {
    const ajv = new _2020_1.default({
        coerceTypes: true,
        useDefaults: true,
        removeAdditional: true,
        formats
    });
    const compileAjvBasedValidateCached = memoize(executable_openapi_parameters_1.compileAjvBasedValidate);
    return (handle, parameters, body, context, info) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const originalPathParameters = (_a = parameters.path) !== null && _a !== void 0 ? _a : {};
        const resolveRef = documentRefResolver !== null && documentRefResolver !== void 0 ? documentRefResolver : (0, openapi_document_local_ref_resolver_1.localRefResolver)(info.document);
        // merge specs
        const parametersSpecs = yield (0, executable_openapi_parameters_1.resolveOperationParametersSpecs)(info.operationObject, info.pathItemObject, resolveRef);
        const pathParametersSpecs = parametersSpecs.filter((p) => p.in === 'path');
        const newPathParameters = {};
        for (const pathParametersSpec of pathParametersSpecs) {
            const { name, schema } = pathParametersSpec;
            let value = originalPathParameters[name];
            // deserialize
            // if value was already not string we can assume it was already
            // processed => we just continue to coerce and validate
            if (typeof value === 'string') {
                const { style = 'simple' } = pathParametersSpec;
                // todo: handle other parameters styles
                if (style !== 'simple') {
                    throw new Error('NotImplemented: missing support for style other than simple (see https://git.io/JD1vU)');
                }
                const tpl = uriTemplates(`{${name}}`);
                value = (_b = tpl.fromUri(value)) === null || _b === void 0 ? void 0 : _b[name];
            }
            // per spec, path parameters are always required
            if (value === undefined) {
                const error = new executable_openapi_parameters_1.InvalidParameterError(name, 'path', 'is required');
                return yield formatErrorResponse(error, info.operationObject.responses);
            }
            // validates and coerce
            if (schema !== undefined) {
                // @ts-expect-error imcompatible options ?
                const validate = compileAjvBasedValidateCached(ajv, schema);
                const { pass, errors, data: newValue } = validate(value);
                if (!pass) {
                    (0, ts_invariant_1.default)(errors, '`validate` should return defined `errors` when `pass` is true');
                    const error = new executable_openapi_parameters_1.InvalidParameterError(name, 'path', errors);
                    return yield formatErrorResponse(error, info.operationObject.responses);
                }
                newPathParameters[name] = newValue;
            }
            else {
                newPathParameters[name] = value;
            }
        }
        return yield handle(Object.assign(Object.assign({}, parameters), { path: newPathParameters }), body, context, info);
    });
};
exports.executableOpenAPIMiddlewarePathParameters = executableOpenAPIMiddlewarePathParameters;
//# sourceMappingURL=middleware.js.map